/*
Copyright (C) 2003-2006 Andrey Nazarov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include <setjmp.h>
#include "qcommon.h"
#include "q_list.h"
#include "q_lex.h"
#include "script_public.h"

typedef enum opcode_e {
	OP_BREAK,
	OP_NOP,

// subrouting operations
	OP_ENTER,
	OP_LEAVE,
	OP_CALL,

// float operations
	OP_MUL_F,
	OP_DIV_F,
	OP_ADD_F,
	OP_SUB_F,
	OP_NEG_F,

	OP_EQ_F,
	OP_NE_F,
	OP_GE_F,
	OP_GT_F,
	OP_LE_F,
	OP_LT_F,

// integer operations
	OP_MUL_I,
	OP_DIV_I,
	OP_MOD_I,
	OP_ADD_I,
	OP_SUB_I,
	OP_NEG_I,

	OP_EQ_I,
	OP_NE_I,
	OP_GE_I,
	OP_GT_I,
	OP_LE_I,
	OP_LT_I,

	OP_BITOR,
	OP_BITAND,
	OP_BITXOR,
	OP_BITINV,

// unary operations
	OP_UMUL_F,
	OP_UDIV_F,
	OP_UADD_F,
	OP_USUB_F,
	OP_UINC_F,
	OP_UDEC_F,

	OP_UMUL_I,
	OP_UDIV_I,
	OP_UMOD_I,
	OP_UADD_I,
	OP_USUB_I,
	OP_UINC_I,
	OP_UDEC_I,

	OP_UBITOR,
	OP_UBITAND,
	OP_UBITXOR,

	OP_UMUL_CF,
	OP_UDIV_CF,
	OP_UADD_C,
	OP_USUB_C,

	OP_UMUL_VF,
	OP_UDIV_VF,
	OP_UADD_V,
	OP_USUB_V,

	OP_UADD_S,

// vector operations
	OP_MUL_VF,
	OP_DIV_VF,
	OP_ADD_V,
	OP_SUB_V,
	OP_NEG_V,
	OP_EQ_V,
	OP_NE_V,

// color operations
	OP_MUL_CF,
	OP_DIV_CF,
	OP_ADD_C,
	OP_SUB_C,
	OP_NEG_C,
	OP_EQ_C,
	OP_NE_C,

// string operations
	OP_ADD_S,
	OP_EQ_S,
	OP_NE_S,
	OP_GE_S,
	OP_GT_S,
	OP_LE_S,
	OP_LT_S,

// stack operations
	OP_INDIRECT,
	OP_INDIRECT_P,

	OP_STORE_I,
	OP_STORE_F,
	OP_STORE_V,
	OP_STORE_C,
	OP_STORE_R,
	OP_STORE_S,
	OP_STORE_P,
	OP_STORE_B,

	OP_STOREP_I,
	OP_STOREP_F,
	OP_STOREP_V,
	OP_STOREP_C,
	OP_STOREP_R,
	OP_STOREP_S,
	OP_STOREP_P,
	OP_STOREP_B,

	OP_LOADP_I,
	OP_LOADP_F,
	OP_LOADP_V,
	OP_LOADP_C,
	OP_LOADP_R,
	OP_LOADP_S,
	OP_LOADP_P,
	OP_LOADP_B,

// branching operations
	OP_IFNOT,
	OP_JUMP,
	OP_NOT,

// conversion operations
	OP_CV_FI,
	OP_CV_IF,
	OP_CV_SI,
	OP_CV_IS,
	OP_CV_SF,
	OP_CV_FS,
	OP_CV_CS,
	OP_CV_BS,
	OP_CV_CVI,
	OP_CV_CVF,
	OP_CV_CVS,
	OP_CV_MI,
	OP_CV_MF,
	OP_CV_MS,

	OP_UNDEFINED
} opcode_t;

typedef struct operator_s {
	char	*string; /* as seen in code */
	char	*name; /* internal name, OP_FOO */
	opcode_t code;
	int		priority;
	lexImmType_t a, b, c;
} operator_t;

typedef struct statement_s {
	opcode_t	op;
	int		param;
	def_t	*a, *b, *c;
	struct statement_s *patchNext;
} statement_t;

operator_t *Script_FindOpcode( int code );

typedef struct {
	statement_t	*statements;
	int			maxStatements;
	int			numStatements;
} statementBuffer_t;

extern statementBuffer_t	stbuffer;
extern statementBuffer_t	*stbuf;

void Script_ParseDef( void );



