/*
Copyright (C) 2003-2006 Andrey Nazarov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

struct def_s;

typedef void (*callback_t)( struct def_s *self, struct def_s *ret );

typedef union {
	int		offset;
	int		integer;
	float	value;
	char	*string;
	vec3_t	vector;
	vec4_t	color;
	void	*pointer;
	callback_t callback;
	vrect_t rect;
	cvar_t	*cvar;
	xmacro_t macro;
} eval_t;

typedef enum {
	EV_VOID,
	EV_INT,
	EV_FLOAT,
	EV_STRING,
	EV_BOOL,
	EV_VECTOR,
	EV_COLOR,
	EV_RECT,
	EV_CVAR,
	EV_MACRO,
	EV_FUNCTION,
	EV_FIELD
} evalType_t;

#define LAST_SIMPLE_TYPE		EV_RECT

typedef struct scope_s {
	list_t defs;
	struct scope_s *parent;
	struct scope_s *next;
} scope_t;

typedef enum {
	DEF_TEMP		= ( 1 << 0 ),
	DEF_IMMEDIATE	= ( 1 << 1 ),
	DEF_POINTER		= ( 1 << 2 ),
	DEF_BUILTIN		= ( 1 << 3 ),
	DEF_CONST		= ( 1 << 4 ),
	DEF_DONTFREE	= ( 1 << 5 )
} defFlags_t;

#define MAX_ARGS	32

typedef struct def_s {
	listElem_t	elem;
	evalType_t	type;
	eval_t		eval;
	defFlags_t	flags;
	evalType_t	auxtype;
	scope_t		*scope;

	/* function types */
	int			numArgs;
	struct def_s *args[MAX_ARGS];
	int			counter;

	char		name[1];
} def_t;

typedef struct {
	char *name;
	evalType_t args[MAX_ARGS];
	evalType_t rettype;
	callback_t callback;
} builtinFunction_t;

#define ARG_S( x )	self->args[x]->eval.string
#define ARG_F( x )	self->args[x]->eval.value
#define ARG_I( x )	self->args[x]->eval.integer
#define ARG_C( x )	self->args[x]->eval.color
#define ARG_R( x )	self->args[x]->eval.rect

void Script_CompileFile( const char *fileName );
def_t *Script_FindFunction( char *name );
void Script_DumpFunction( def_t *function );
void Script_Execute( eval_t *ret, def_t *function, ... );
void Script_CreateBuiltins( builtinFunction_t *funcs );
def_t *Script_CreateDef( const char *name, evalType_t type );
void Script_Parse( void );
void Script_Free( void );



