/*
Copyright (C) 2003-2006 Andrey Nazarov

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

//
// cl_defs.c
//

#include "client.h"
#include <setjmp.h>
#include "q_lex.h"

list_t	cl_entdefs;

void CL_ClearEntdefs( void ) {
	listElem_t *elem;
	entdef_t *entdef;

	for( elem = cl_entdefs.first; elem; elem = elem->next ) {
		entdef = ( entdef_t * )elem;
		entdef->modelindex = 0;
		entdef->visualindex = 0;
	}
}

void CL_RegisterEntdefs( void ) {
	listElem_t *elem;
	entdef_t *entdef;
	char buffer[16];

	for( elem = cl_entdefs.first; elem; elem = elem->next ) {
		entdef = ( entdef_t * )elem;
		if( !entdef->crosshairIndex ) {
			continue;
		}
		Com_sprintf( buffer, sizeof( buffer ), "ch%i", entdef->crosshairIndex );
		entdef->crosshairHandle = ref.RegisterPic( buffer );
		if( !entdef->crosshairHandle ) {
			Com_WPrintf( "Couldn't register crosshair %d\n", entdef->crosshairIndex );
			continue;
		}
		ref.DrawGetPicSize( &entdef->crosshairWidth, &entdef->crosshairHeight,
			entdef->crosshairHandle );
	}
}

void CL_AssignEntdef( int index, const char *string ) {
	listElem_t *elem;
	entdef_t *entdef;

	for( elem = cl_entdefs.first; elem; elem = elem->next ) {
		entdef = ( entdef_t * )elem;
		if( entdef->model[0] && !strncmp( entdef->model, string, strlen( entdef->model ) ) ) {
			entdef->modelindex = index;
			break;
		}
		if( entdef->visual[0] && !strncmp( entdef->visual, string, strlen( entdef->visual ) ) ) {
			entdef->visualindex = index;
			break;
		}
	}
}

static void CL_ParseEntdef( void ) {
	char icon[MAX_QPATH];
	char model[MAX_QPATH];
	char visual[MAX_QPATH];
	char name[MAX_QPATH];
	int crosshair;
	float weight;
	entdef_t *entdef;

	icon[0] = 0;
	model[0] = 0;
	visual[0] = 0;
	name[0] = 0;
	crosshair = 0;
	weight = 1;

	Lex_Expect( "entdef" );
	Lex_Expect( "{" );
	while( !Lex_Check( "}" ) ) {
		if( lex.tokenType != TT_NAME ) {
			Lex_Error( "Expected keyword name" );
		}
		if( Lex_Check( "icon" ) ) {
			Lex_ExpectImmediate( IT_STRING );
			Q_strncpyz( icon, lex_immediate.string, sizeof( icon ) );
		} else if( Lex_Check( "model" ) ) {
			Lex_ExpectImmediate( IT_STRING );
			Q_strncpyz( model, lex_immediate.string, sizeof( model ) );
		} else if( Lex_Check( "visual" ) ) {
			Lex_ExpectImmediate( IT_STRING );
			Q_strncpyz( visual, lex_immediate.string, sizeof( visual ) );
		} else if( Lex_Check( "name" ) ) {
			Lex_ExpectImmediate( IT_STRING );
			Q_strncpyz( name, lex_immediate.string, sizeof( name ) );
		} else if( Lex_Check( "crosshair" ) ) {
			Lex_ExpectImmediate( IT_INT );
			crosshair = lex_immediate.integer;
		} else if( Lex_Check( "weight" ) ) {
			Lex_ExpectImmediate( IT_INT|IT_FLOAT );
			if( lex_immediateType == IT_INT ) {
				weight = lex_immediate.integer;
			} else {
				weight = lex_immediate.value;
			}
		} else {
			Lex_Error( "Unknown keyword '%s'", lex.token );
		}

		Lex_Expect( ";" );
	}

	if( !model[0] && !visual[0] ) {
		Com_WPrintf( "line %d: no model set, entdef ignored.\n", lex.currentLine );
		return;
	}
	if( !name[0] ) {
		Com_WPrintf( "line %d: no name set, entdef ignored.\n", lex.currentLine );
		return;
	}
	if( weight <= 0 || weight > 2 ) {
		Com_WPrintf( "line %d: bad weight set, entdef ignored.\n", lex.currentLine );
		return;
	}
	if( crosshair < 0 ) {
		Com_WPrintf( "line %d: bad crosshair set, entdef ignored.\n", lex.currentLine );
		return;
	}
	if( crosshair && !visual[0] ) {
		Com_WPrintf( "line %d: crosshair without visual set, crosshair disabled.\n", lex.currentLine );
		crosshair = 0;
	}

	Z_Reserve( sizeof( *entdef ) +
		strlen( name ) +
		strlen( model ) +
		strlen( visual ) +
		strlen( icon ) + 4 );

	entdef = Z_ReservedAlloc( sizeof( *entdef ) );
	entdef->name = Z_ReservedCopyString( name );
	entdef->model = Z_ReservedCopyString( model );
	entdef->visual = Z_ReservedCopyString( visual );
	entdef->icon = Z_ReservedCopyString( icon );
	entdef->crosshairIndex = crosshair;
	entdef->weight = 1.0f / weight;

	List_Append( &cl_entdefs, entdef );
	
}

static void CL_NearestItem_m( char *buffer, int bufferSize ) {
	int i, num;
	entity_state_t *ent;
	vec3_t dir;
	float dist, mindist;
	listElem_t *elem;
	entdef_t *entdef, *closest;

	Q_strncpyz( buffer, "unknown", bufferSize );

	if( cls.state != ca_active ) {
		return;
	}

	mindist = 9999;
	closest = NULL;
	for( i = 0; i < cl.frame.numEntityStates; i++ ) {
		num = ( cl.frame.firstEntityState + i ) & PARSE_ENTITIES_MASK;
		ent = &cl.entityStates[num];

		if( ent->number == cl.frame.ps.clientNum + 1 ) {
			continue;
		}

		if( ent->solid == 31 ) {
			continue;
		}

		if( ent->modelindex == 0 || ent->modelindex == 255 ) {
			continue;
		}

		for( elem = cl_entdefs.first; elem; elem = elem->next ) {
			entdef = ( entdef_t * )elem;
			if( entdef->modelindex == ent->modelindex ) {
				break;
			}
		}

		if( !elem ) {
			continue;
		}

		VectorSubtract( cl.refdef.vieworg, ent->origin, dir );
		dist = VectorLength( dir ) * entdef->weight;
		if( mindist > dist ) {
			mindist = dist;
			closest = entdef;
		}
	}

	if( !closest ) {
		return;
	}

	Q_strncpyz( buffer, closest->name, bufferSize );
}

static void CL_Weapon_m( char *buffer, int bufferSize ) {
	if( !cl.weapondef ) {
		Q_strncpyz( buffer, "unknown", bufferSize );
		return;
	}

	Q_strncpyz( buffer, cl.weapondef->name, bufferSize );	
}

void CL_SetWeaponDef( void ) {
	listElem_t *elem;
	entdef_t *entdef;

	cl.weapondef = NULL;

	if( !cl.frame.ps.ps.gunindex ) {
		return;
	}

	for( elem = cl_entdefs.first; elem; elem = elem->next ) {
		entdef = ( entdef_t * )elem;
		if( entdef->visualindex == cl.frame.ps.ps.gunindex ) {
			cl.weapondef = entdef;
			break;
		}
	}
}

void CL_FreeEntdefs_f( void ) {
	List_Free( &cl_entdefs );
}

void CL_LoadEntdefs_f( void ) {
	char buffer[MAX_QPATH];
	char *data;
	int i;

	if( Cmd_Argc() < 2 ) {
		Com_Printf( "Usage: %s <filename>\n", Cmd_Argv( 0 ) );
		return;
	}

	Cmd_ArgvBuffer( 1, buffer, sizeof( buffer ) );
	COM_DefaultExtension( buffer, ".def", sizeof( buffer ) );

	FS_LoadFile( buffer, ( void ** )&data );
	if( !data ) {
		Com_Printf( "Couldn't load %s\n", buffer );
		return;
	}

	CL_FreeEntdefs_f();

	Lex_BeginParse( data );

	if( _setjmp( lex_jmpbuf ) ) {
		FS_FreeFile( data );
		return;
	}

	while( lex.tokenType != TT_EOF ) {
		CL_ParseEntdef();
	}

	FS_FreeFile( data );

	if( cls.state >= ca_connected ) {
		for( i = 1; i < MAX_MODELS; i++ ) {
			if( cl.configstrings[CS_MODELS+i][0] ) {
				CL_AssignEntdef( i, cl.configstrings[CS_MODELS+i] );
			}
		}
	}
	CL_RegisterEntdefs();

	i = List_CountElems( &cl_entdefs );
	Com_Printf( "Loaded %d entdefs from %s\n", i, buffer );

}

void CL_InitEntdefs( void ) {
	Cmd_AddCommand( "loadentdefs", CL_LoadEntdefs_f );
	Cmd_AddCommand( "freeentdefs", CL_FreeEntdefs_f );

	Cmd_AddMacro( "cl_item", CL_NearestItem_m );
	Cmd_AddMacro( "cl_weapon", CL_Weapon_m );
}
